/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.crop.model;

import java.awt.Color;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import nl.cloudfarming.client.model.AgroURI;
import org.openide.util.Lookup;

/**
 * <P>An crop instance represents a Crop with it's classification and it's variety</p>
 * <p>It holds links to the original crop from which it was created via the uri of the crop</p>
 * @author Wytse Visser
 */
public class CropInstance implements Serializable {

    private transient PropertyChangeSupport propertyChangeSupport;
    private Color color;
    public static final String PROP_COLOR = "color";
    private AgroURI cropURI;
//    private AgroURI varietyURI;
//    private AgroURI classificationURI;
    private Variety variety;
    private Classification classification;

    /**
     * <p>Creates a CropInstance with a crop a variety and a classification</p>
     * The passed crop needs to have a URI set otherwise an {@link IllegalArgumentException} is thrown  
     * @param crop
     * @param variety
     * @param classification 
     */
    public CropInstance(Crop crop, Variety variety, Classification classification) {
        if(crop.getURI() == null) throw new IllegalArgumentException("Crop needs an URI set");
        setCrop(crop);
        setVariety(variety);
        setClassification(classification);
    }

    public final void setCropDefinition(CropDefinition cropDefinition) {
        setColor(cropDefinition.getColor());
        setCrop(cropDefinition.getCrop());
    }

    public final void setCrop(Crop crop) {
        this.cropURI = crop.getURI();
    }

    /**
     * Tries to find the crop of this CropInstance via the uri of the crop
     * @return 
     */
    public Crop getCrop() {
        if (cropURI != null) {
            CropManager manager = Lookup.getDefault().lookup(CropManager.class);
            return manager.find(cropURI);
        }
        return null;
    }

    public final void setVariety(Variety variety) {
        this.variety = variety; // FIXME: URIfy + add listener -WV
    }

    public Variety getVariety() {
        return variety; // FIXME:URIfy
//        VarietyManager varietyManager = Lookup.getDefault().lookup(VarietyManager.class);
//        return varietyManager.find(varietyURI);
    }

    public final void setClassification(Classification classification) {
        this.classification = classification;// FIXME: URIfy + add listener -WV
    }

    public Classification getClassification() {
        return classification; // FIXME: URIfy 
    }

    public Color getColor() {
        return color;
    }

    public void setColor(Color color) {
        Color oldColor = this.color;
        this.color = color;
        getPropertyChangeSupport().firePropertyChange(PROP_COLOR, oldColor, color);
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        if (getCrop() != null) {
            getCrop().addPropertyChangeListener(listener);
        }
        getPropertyChangeSupport().addPropertyChangeListener(listener);
    }

    public String getName() {
        return getCrop().getName();
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(listener);
    }

    protected PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }
}
